﻿using System.Collections.Generic;
using Ozeki.VoIP;

namespace MediaGatewayClients.MediaGateway
{
    /// <summary>
    /// SIP account container class.
    /// </summary>
    class AccountPool
    {
        Queue<string> accounts;

        /// <summary>
        /// Create a new instace of SIPAccountPool.
        /// </summary>
        public AccountPool()
        {
            accounts = new Queue<string>();
        }

        /// <summary>
        /// Gets the next free SIP account.
        /// </summary>
        public string GetAccount()
        {
            lock (accounts)
            {
                if (accounts.Count > 0)
                {
                    string sipAccount = accounts.Dequeue();
                    return sipAccount;
                }
            }

            return null;
        }

        /// <summary>
        /// Adds a SIP account to the accounts queue.
        /// </summary>
        public void AddAccount(string account)
        {
            lock (accounts)
            {
                accounts.Enqueue(account);
            }
        }
    }
}
